// Flutter imports:
import 'package:flutter/cupertino.dart';

// Project imports:
import 'package:zego_uikit/src/components/audio_video/audio_video_view.dart';
import 'package:zego_uikit/src/components/audio_video_container/layout_gallery.dart';
import 'package:zego_uikit/src/services/services.dart';
import 'layout.dart';
import 'layout_picture_in_picture.dart';

/// container of audio video view,
/// it will layout views by layout mode and config
class ZegoAudioVideoContainer extends StatefulWidget {
  const ZegoAudioVideoContainer({
    Key? key,
    required this.layout,
    this.foregroundBuilder,
    this.backgroundBuilder,
  }) : super(key: key);

  final ZegoLayout layout;

  /// foreground builder of audio video view
  final AudioVideoViewForegroundBuilder? foregroundBuilder;

  /// background builder of audio video view
  final AudioVideoViewBackgroundBuilder? backgroundBuilder;

  @override
  State<ZegoAudioVideoContainer> createState() =>
      _ZegoAudioVideoContainerState();
}

class _ZegoAudioVideoContainerState extends State<ZegoAudioVideoContainer> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<List<ZegoUIKitUser>>(
      stream: ZegoUIKit().getUserListStream(),
      builder: (context, snapshot) {
        List<ZegoUIKitUser> userList = snapshot.data ?? [];
        userList
            .removeWhere((user) => user.id == ZegoUIKit().getLocalUser().id);

        if (widget.layout is ZegoLayoutPictureInPictureConfig) {
          return pictureInPictureLayout(userList);
        } else if (widget.layout is ZegoLayoutGalleryConfig) {
          return galleryLayout(userList);
        }

        assert(false, "Unimplemented layout");
        return Container();
      },
    );
  }

  /// picture in picture
  Widget pictureInPictureLayout(List<ZegoUIKitUser> remoteUserList) {
    /// the last user entered is used as the remote user
    var remoteUser = remoteUserList.isEmpty ? null : remoteUserList.last;

    return ZegoLayoutPictureInPicture(
      layoutConfig: widget.layout as ZegoLayoutPictureInPictureConfig,
      backgroundBuilder: widget.backgroundBuilder,
      foregroundBuilder: widget.foregroundBuilder,
      localUser: ZegoUIKit().getLocalUser(),
      remoteUser: remoteUser,
    );
  }

  /// gallery
  Widget galleryLayout(List<ZegoUIKitUser> remoteUserList) {
    List<ZegoUIKitUser> reversedRemoteUserList =
        List.from(remoteUserList.reversed);
    return ZegoLayoutGallery(
      layoutConfig: widget.layout as ZegoLayoutGalleryConfig,
      backgroundBuilder: widget.backgroundBuilder,
      foregroundBuilder: widget.foregroundBuilder,
      localUser: ZegoUIKit().getLocalUser(),
      remoteUser: reversedRemoteUserList,
    );
  }
}
